import VueSyncersFeathers from 'vue-syncers-feathers'
import VueFeathers from 'vue-feathers'

( ->

  plugin = (Vue, options) ->

    Vue.use VueSyncersFeathers, options
    feathers = options.feathers
    Vue.use VueFeathers, feathers

    Vue::$feathers = feathers

    Vue::_getSyncer = (syncerData) ->
      for syncerName, syncer of @_syncers
        if syncerData is syncer.state
          return {syncerName, syncer}
      return {
        syncerName: null
        syncer: null
      }

    Vue::$put = (syncerData, mapToPut, options = {}) ->

      new Promise (resolve, reject) =>
        {syncerName, syncer} = @_getSyncer syncerData
        if not syncer?
          reject new Error "Wrong syncer."
          return
        if not syncer.filters?.id? # is not ItemSyncer
          reject new Error "Cannot $put on a collection syncer: #{syncerName}."
          return

        if options.optimistic
          syncerData = @[syncerName] or {}
          oldSyncerData = JSON.parse JSON.stringify syncerData
          for key, value of mapToPut
            console.log key, value, syncerData, syncerName
            @$set syncerData, key, value
          console.log 'optimistic(buggy): ', syncerData

        syncer.saving = true
        @$feathers.service syncer.service.path
        .patch syncer.filters.id, mapToPut

        .then (data) =>
          syncer.saving = false
          @$emit 'syncer-saved', syncer.service.path
          resolve data

        .catch (error) => # FIXME: cannot restore old data when connection lost or partial failure

          if options.optimistic
            for key, value of mapToPut
              if oldSyncerData[key]
                @$set syncerData, key, oldSyncerData[key]
              else
                @$delete syncerData, key
            console.log 'optimistic failed, restored(buggy): ', syncerData

          syncer.saving = false
          @$emit 'syncer-error', syncer.service.path, error
          reject error

    for method in ['create', 'update', 'patch', 'remove']
      do (method) =>

        Vue::["$#{method}"] = (syncerData, args...) ->

          new Promise (resolve, reject) =>
            {syncerName, syncer} = @_getSyncer syncerData
            if not syncer?
              reject new Error "Wrong syncer."
              return
            if syncer.filters?.id? # is ItemSyncer
              reject new Error "Cannot #{method} on an item syncer: #{syncerName}."
              return

            syncer.saving = true
            service = @$feathers.service syncer.service.path
            service[method] args...

            .then (data) =>
              syncer.saving = false
              @$emit 'syncer-saved', syncer.service.path

              params = if method in ['create', 'remove'] then args[1] else args[2]
              if params?.reload
                syncer._loadNewState()

              resolve data

            .catch (error) =>
              syncer.saving = false
              @$emit 'syncer-error', syncer.service.path, error
              reject error

    Vue.mixin

      created: ->

        return unless @_syncers?

        @$syncers = @_syncers
        Vue.util.defineReactive @, '$savingSyncers', false

        for syncerName, syncer of @_syncers
          @$data[syncerName] = syncer
          Vue.util.defineReactive syncer, 'saving', false

        @$watch ->
          for syncerName, syncer of @_syncers
            if syncer.saving
              return true
          return false
        , (value) =>
          @$savingSyncers = value
        ,
          # sync: true
          immediate: true

  if Vue? then Vue.use plugin
  if typeof exports is 'object' and typeof module is 'object'
    module.exports = plugin
  else if typeof define is 'function' and define.amd
    define -> plugin
  else if window?
    window.VueFeathersSync = plugin

)()