var slice = [].slice;

import VueSyncersFeathers from 'vue-syncers-feathers';

import VueFeathers from 'vue-feathers';

(function () {
	var plugin;
	plugin = function (Vue, options) {
		var feathers, fn, i, len, method, ref;
		Vue.use(VueSyncersFeathers, options);
		feathers = options.feathers;
		Vue.use(VueFeathers, feathers);
		Vue.prototype.$feathers = feathers;
		Vue.prototype._getSyncer = function (syncerData) {
			var ref, syncer, syncerName;
			ref = this._syncers;
			for (syncerName in ref) {
				syncer = ref[syncerName];
				if (syncerData === syncer.state) {
					return {
						syncerName: syncerName,
						syncer: syncer
					};
				}
			}
			return {
				syncerName: null,
				syncer: null
			};
		};
		Vue.prototype.$put = function (syncerData, mapToPut, options) {
			if (options == null) {
				options = {};
			}
			return new Promise((function (_this) {
				return function (resolve, reject) {
					var key, oldSyncerData, ref, ref1, syncer, syncerName, value;
					ref = _this._getSyncer(syncerData), syncerName = ref.syncerName, syncer = ref.syncer;
					if (syncer == null) {
						reject(new Error("Wrong syncer."));
						return;
					}
					if (((ref1 = syncer.filters) != null ? ref1.id : void 0) == null) {
						reject(new Error("Cannot $put on a collection syncer: " + syncerName + "."));
						return;
					}
					if (options.optimistic) {
						syncerData = _this[syncerName] || {};
						oldSyncerData = JSON.parse(JSON.stringify(syncerData));
						for (key in mapToPut) {
							value = mapToPut[key];
							console.log(key, value, syncerData, syncerName);
							_this.$set(syncerData, key, value);
						}
						console.log('optimistic(buggy): ', syncerData);
					}
					syncer.saving = true;
					return _this.$feathers.service(syncer.service.path).patch(syncer.filters.id, mapToPut).then(function (data) {
						syncer.saving = false;
						_this.$emit('syncer-saved', syncer.service.path);
						return resolve(data);
					})["catch"](function (error) {
						if (options.optimistic) {
							for (key in mapToPut) {
								value = mapToPut[key];
								if (oldSyncerData[key]) {
									_this.$set(syncerData, key, oldSyncerData[key]);
								} else {
									_this.$delete(syncerData, key);
								}
							}
							console.log('optimistic failed, restored(buggy): ', syncerData);
						}
						syncer.saving = false;
						_this.$emit('syncer-error', syncer.service.path, error);
						return reject(error);
					});
				};
			})(this));
		};
		ref = ['create', 'update', 'patch', 'remove'];
		fn = (function (_this) {
			return function (method) {
				return Vue.prototype["$" + method] = function () {
					var args, syncerData;
					syncerData = arguments[0], args = 2 <= arguments.length ? slice.call(arguments, 1) : [];
					return new Promise((function (_this) {
						return function (resolve, reject) {
							var ref1, ref2, service, syncer, syncerName;
							ref1 = _this._getSyncer(syncerData), syncerName = ref1.syncerName, syncer = ref1.syncer;
							if (syncer == null) {
								reject(new Error("Wrong syncer."));
								return;
							}
							if (((ref2 = syncer.filters) != null ? ref2.id : void 0) != null) {
								reject(new Error("Cannot " + method + " on an item syncer: " + syncerName + "."));
								return;
							}
							syncer.saving = true;
							service = _this.$feathers.service(syncer.service.path);
							return service[method].apply(service, args).then(function (data) {
								var params;
								syncer.saving = false;
								_this.$emit('syncer-saved', syncer.service.path);
								params = method === 'create' || method === 'remove' ? args[1] : args[2];
								if (params != null ? params.reload : void 0) {
									syncer._loadNewState();
								}
								return resolve(data);
							})["catch"](function (error) {
								syncer.saving = false;
								_this.$emit('syncer-error', syncer.service.path, error);
								return reject(error);
							});
						};
					})(this));
				};
			};
		})(this);
		for (i = 0, len = ref.length; i < len; i++) {
			method = ref[i];
			fn(method);
		}
		return Vue.mixin({
			created: function () {
				var ref1, syncer, syncerName;
				if (this._syncers == null) {
					return;
				}
				this.$syncers = this._syncers;
				Vue.util.defineReactive(this, '$savingSyncers', false);
				ref1 = this._syncers;
				for (syncerName in ref1) {
					syncer = ref1[syncerName];
					this.$data[syncerName] = syncer;
					Vue.util.defineReactive(syncer, 'saving', false);
				}
				return this.$watch(function () {
					var ref2;
					ref2 = this._syncers;
					for (syncerName in ref2) {
						syncer = ref2[syncerName];
						if (syncer.saving) {
							return true;
						}
					}
					return false;
				}, (function (_this) {
					return function (value) {
						return _this.$savingSyncers = value;
					};
				})(this), {
					immediate: true
				});
			}
		});
	};
	if (typeof Vue !== "undefined" && Vue !== null) {
		Vue.use(plugin);
	}
	if (typeof exports === 'object' && typeof module === 'object') {
		return module.exports = plugin;
	} else if (typeof define === 'function' && define.amd) {
		return define(function () {
			return plugin;
		});
	} else if (typeof window !== "undefined" && window !== null) {
		return window.VueFeathersSync = plugin;
	}
})();